@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -364,6 +364,11 @@ popup_attach( GtkWidget *host, GtkWidget *popup, void *data )
 	guint sid;
 
 	popup_link( host, popup, data );
+
+	/* We can't just use gtk_menu_attach_to_widget(), since that can only
+	 * attach a menu to a single widget. We want to be able to attach a
+	 * single menu to meny widgets.
+	 */
         sid = gtk_signal_connect( GTK_OBJECT( host ), "event",
                 GTK_SIGNAL_FUNC( popup_handle_event ), NULL );
 
@@ -871,6 +876,19 @@ typedef struct {
 	void *client;
 } FiledropInfo;
 
+static gboolean
+filedrop_trigger( FiledropInfo *fdi, const char *path )
+{
+	char buf[FILENAME_MAX];
+	gboolean result;
+
+	im_strncpy( buf, path, FILENAME_MAX );
+	path_compact( buf );
+	result = fdi->fn( fdi->client, buf );
+
+	return( result );
+}
+
 static void
 filedrop_drag_data_received( GtkWidget *widget, 
 	GdkDragContext *context,
@@ -897,12 +915,12 @@ filedrop_drag_data_received( GtkWidget *widget,
 		sPath = g_strndup( pFrom, pTo - pFrom );
 
 #if !GLIB_CHECK_VERSION (2,0,0)
-		result = fdi->fn( fdi->client, sPath );
+		result = filedrop_trigger( fdi, sPath );
 #else
 		/* format changed with Gtk+1.3, use conversion 
 		 */
 		pFrom = g_filename_from_uri( sPath, NULL, &error );
-		result = fdi->fn( fdi->client, pFrom );
+		result = filedrop_trigger( fdi, pFrom );
 		g_free( pFrom );
 #endif
 
@@ -1107,7 +1125,7 @@ image_new_from_file( const char *name )
 	GtkWidget *image;
 	char *file;
 
-	if( (file = path_find_file( PATH_SEARCH, name )) ) {
+	if( (file = path_find_file( name )) ) {
 		image = (GtkWidget *) callv_string_filename( 
 			(callv_string_fn) gtk_image_new_from_file, 
 			file, NULL, NULL, NULL );
@@ -1154,8 +1172,9 @@ void
 text_view_set_text( GtkTextView *text_view, 
 	const char *text, gboolean editable )
 {
-	gtk_text_buffer_set_text( gtk_text_view_get_buffer( text_view ),
-		text ? text : "", -1 );
+	GtkTextBuffer *text_buffer = gtk_text_view_get_buffer( text_view );
+
+	gtk_text_buffer_set_text( text_buffer, text ? text : "", -1 );
 
 	gtk_text_view_set_editable( text_view, editable );
 	gtk_text_view_set_cursor_visible( text_view, editable );
