@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -184,19 +184,20 @@ matrix_edit( GtkWidget *parent, Model *model )
 	Matrix *matrix = MATRIX( model );
 	MatrixEdit *eds = INEW( NULL, MatrixEdit );
 	GtkWidget *idlg;
-	char txt[256];
-	VipsBuf buf = VIPS_BUF_STATIC( txt );
 
 	eds->matrix = matrix;
 
 	idlg = idialog_new();
-	iwindow_set_title( IWINDOW( idlg ), 
-		_( "Edit Matrix \"%s\"" ), vips_buf_all( &buf ) );
+	iwindow_set_title( IWINDOW( idlg ), _( "Edit %s %s" ),
+		IOBJECT_GET_CLASS_NAME( model ),
+		IOBJECT( HEAPMODEL( model )->row )->name );
 	idialog_set_build( IDIALOG( idlg ), 
 		(iWindowBuildFn) matrix_buildedit, eds, NULL, NULL );
 	idialog_set_callbacks( IDIALOG( idlg ), 
 		iwindow_true_cb, NULL, idialog_free_client, eds );
-	idialog_add_ok( IDIALOG( idlg ), matrix_done_cb, _( "Set Matrix" ) );
+	idialog_add_ok( IDIALOG( idlg ), 
+		matrix_done_cb, _( "Set %s" ), 
+		IOBJECT_GET_CLASS_NAME( model ) );
 	iwindow_set_parent( IWINDOW( idlg ), parent );
 	idialog_set_iobject( IDIALOG( idlg ), IOBJECT( model ) );
 	iwindow_build( IWINDOW( idlg ) );
@@ -210,11 +211,18 @@ matrix_graphic_save( Classmodel *classmodel,
 {
 	Matrix *matrix = MATRIX( classmodel );
 	DOUBLEMASK *dmask;
+	char buf[FILENAME_MAX];
 
 	if( !(dmask = matrix_model_to_dmask( matrix )) ) 
 		return( FALSE );
 
-	if( im_write_dmask_name( dmask, filename ) ) {
+	/* We don't want $VAR etc. in the filename we pass down to the file
+	 * ops.
+	 */
+	im_strncpy( buf, filename, FILENAME_MAX );
+	path_expand( buf );
+
+	if( im_write_dmask_name( dmask, buf ) ) {
 		error_vips_all();
 		IM_FREEF( im_free_dmask, dmask );
 		return( FALSE );
@@ -237,7 +245,13 @@ matrix_graphic_replace( Classmodel *classmodel,
 	char txt[MAX_STRSIZE];
 	VipsBuf buf = VIPS_BUF_STATIC( txt );
 
-	if( !(dmask = im_read_dmask( filename )) ) {
+	/* We don't want $VAR etc. in the filename we pass down to the file
+	 * ops.
+	 */
+	im_strncpy( txt, filename, FILENAME_MAX );
+	path_expand( txt );
+
+	if( !(dmask = im_read_dmask( txt )) ) {
 		error_vips_all();
 		return( FALSE );
 	}
@@ -279,6 +293,7 @@ static void
 matrix_class_init( MatrixClass *class )
 {
 	GObjectClass *gobject_class = (GObjectClass *) class;
+	iObjectClass *iobject_class = (iObjectClass *) class;
 	ModelClass *model_class = (ModelClass *) class;
 	ClassmodelClass *classmodel_class = (ClassmodelClass *) class;
 
@@ -291,6 +306,8 @@ matrix_class_init( MatrixClass *class )
 	 */
 	gobject_class->finalize = matrix_finalize;
 
+	iobject_class->user_name = _( "Matrix" );
+
 	model_class->view_new = matrix_view_new;
 	model_class->edit = matrix_edit;
 
