@@ -15,9 +15,9 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
  */
 
@@ -102,7 +102,7 @@ watchgroup_get_type( void )
 }
 
 Watchgroup *
-watchgroup_new( Workspacegroup *workspacegroup, const char *name )
+watchgroup_new( Workspaceroot *workspaceroot, const char *name )
 {
 	Watchgroup *watchgroup = WATCHGROUP( 
 		g_object_new( TYPE_WATCHGROUP, NULL ) );
@@ -111,7 +111,7 @@ watchgroup_new( Workspacegroup *workspacegroup, const char *name )
 	 */
 	watchgroup->name = name;
 
-	watchgroup->workspacegroup = workspacegroup;
+	watchgroup->workspaceroot = workspaceroot;
 	icontainer_set_hash( ICONTAINER( watchgroup ) );
 
 	return( watchgroup );
@@ -125,10 +125,10 @@ watchgroup_get_workspace( Watchgroup *watchgroup )
 	Compile *compile;
 	Symbol *sym;
 
-	if( !watchgroup->workspacegroup->sym )
+	if( !watchgroup->workspaceroot->sym )
 		return( NULL );
 
-	compile = watchgroup->workspacegroup->sym->expr->compile;
+	compile = watchgroup->workspaceroot->sym->expr->compile;
 
 	if( !(sym = compile_lookup( compile, watchgroup->name )) ||
 		!sym->expr->compile ||
@@ -145,7 +145,8 @@ watchgroup_save( Watchgroup *watchgroup )
 	Workspace *ws;
 
 	if( (ws = watchgroup_get_workspace( watchgroup )) ) {
-		Filemodel *filemodel = FILEMODEL( ws );
+		Workspacegroup *wsg = workspace_get_workspacegroup( ws );
+		Filemodel *filemodel = FILEMODEL( wsg );
 
 		if( filemodel->modified ) {
 			symbol_recalculate_all();
@@ -153,7 +154,7 @@ watchgroup_save( Watchgroup *watchgroup )
 			/* Ignore error returns ... hmm! Tricky: we can come
 			 * here during shutdown.
 			 */
-			(void) filemodel_save_all( filemodel, 
+			(void) filemodel_top_save( filemodel, 
 				filemodel->filename );
 
 			filemodel_set_modified( filemodel, FALSE );
@@ -178,12 +179,12 @@ watchgroup_dirty( Watchgroup *watchgroup )
 
 	/* Find the preferences workspace.
 	 */
-	if( (ws = watchgroup_get_workspace( watchgroup )) ) {
-		Filemodel *filemodel = FILEMODEL( ws );
+	if( (ws = watchgroup_get_workspace( watchgroup )) ) { 
+		Workspacegroup *wsg = workspace_get_workspacegroup( ws );
 
 		/* Mark ws dirty, start save timer.
 		 */
-		filemodel_set_modified( filemodel, TRUE );
+		filemodel_set_modified( FILEMODEL( wsg ), TRUE );
 
 		IM_FREEF( g_source_remove, watchgroup->auto_save_timeout );
 		watchgroup->auto_save_timeout = g_timeout_add( 1000, 
@@ -433,7 +434,18 @@ watch_relink_all( void )
 void
 watch_vset( Watch *watch, const char *fmt, va_list args )
 {
-	if( watch->row && watch->row->child_rhs && 
+	Watchgroup *watchgroup = WATCHGROUP( ICONTAINER( watch )->parent );
+
+	Workspace *ws;
+
+	/* In case we try to set after prefs has gone.
+	 */
+	if( !(ws = watchgroup_get_workspace( watchgroup )) ||
+		ws->in_dispose )
+		return;
+
+	if( watch->row && 
+		watch->row->child_rhs && 
 		watch->row->child_rhs->itext ) {
 		iText *itext = ITEXT( watch->row->child_rhs->itext );
 		char buf[256];
@@ -447,7 +459,9 @@ watch_vset( Watch *watch, const char *fmt, va_list args )
 #endif /*DEBUG*/
 
 			itext_set_edited( itext, TRUE );
-			expr_dirty( watch->row->sym->expr, link_serial_new() );
+			if( watch->row->sym )
+				expr_dirty( watch->row->sym->expr, 
+					link_serial_new() );
 			watchgroup_dirty( 
 				WATCHGROUP( ICONTAINER( watch )->parent ) );
 		}
@@ -580,7 +594,7 @@ watch_int_update( Watch *watch )
 {
 	WatchInt *watch_int = WATCH_INT( watch );
 	Expr *expr = watch->row->expr;
-	PElement *root = &watch->row->expr->root;
+	PElement *root;
 
 #ifdef DEBUG
 	printf( "watch_int_update: %s\n", NN( IOBJECT( watch )->name ) );
@@ -674,7 +688,8 @@ watch_int_get( Watchgroup *watchgroup, const char *name, int fallback )
 	Watch *watch;
 	void *value;
 
-	if( !watchgroup || !name )
+	if( !watchgroup || 
+		!name )
 		return( fallback );
 
 	if( !(watch = watch_find( watchgroup, name )) )
